//
//  RedirectResolver.m
//  pay
//
//  Created by Jasmin Suljic on 21/12/16.
//  Copyright © 2016 Webteh.hr. All rights reserved.
//

#import "RedirectResolver.h"

const int STARTED_TRANSACTION = 0;
const int TRANSACTION_SUCCESS = 1;
const int TRANSACTION_FAILURE = 2;
const int TIMEOUT = 3;
const int UNKNOWN = 4;
const int THREE_DS = 5;
const int STARTED_TRANSACTION_3DS = 6;
const int IGNORE = 2121;


@interface RedirectResolver()

@property(strong,nonatomic) TransactionLifecycleAdapter *transactionLifecycleAdapter;
@property bool transactionInProgress;
@property long timePassed;
@property long timeout;
@property long timeoutHandlerPeriod;
@property(strong, nonatomic) NSTimer* timer;

-(void) stopTimeoutTracking;
-(void) startTimeoutTracking;

@end

@implementation RedirectResolver

-(id) init: (TransactionLifecycleAdapter *) adapter
   timeout: (long) timeout
{
    self.transactionLifecycleAdapter = adapter;
    self.transactionInProgress = false;
    _timePassed = 0;
    _timeout = timeout;
    _timeoutHandlerPeriod = 500;
    return [self init];
}

-(int) resolve:(NSString *)request{
    if ([request rangeOfString:@"/api/add-card"].location != NSNotFound || [request rangeOfString:@"/api/pay-cvv"].location != NSNotFound) {
        return STARTED_TRANSACTION;
    } else if([request rangeOfString:@"/api/pares"].location != NSNotFound){
        return STARTED_TRANSACTION_3DS;
    } else if([request rangeOfString:@"/api/success"].location != NSNotFound) {
        return TRANSACTION_SUCCESS;
    } else if([request rangeOfString:@"/api/error"].location != NSNotFound){
        return TRANSACTION_FAILURE;
    } else if([request rangeOfString:@"/default/add-card"].location != NSNotFound || [request rangeOfString:@"/default/pay-cvv"].location != NSNotFound){
        return IGNORE;
    } else if([request rangeOfString:@"/default/3ds"].location != NSNotFound){
        return THREE_DS;
    }
    else{
        return UNKNOWN;
    }
}

-(void) notifyDelegate:(int)resolvedCode request:(NSString *)request{
    NSLog(@"notifyDelegate: %@", request);
    NSLog(@"notifyDelegate: resolvedCode: %d", resolvedCode);
    NSArray *testArray = request == nil ? nil: [request componentsSeparatedByString:@"/"];
    NSString *inputParam = (testArray == nil || testArray.count == 0)? nil: testArray[testArray.count - 1];
    
    switch (resolvedCode) {
        case STARTED_TRANSACTION:
        case STARTED_TRANSACTION_3DS:
            _transactionInProgress = true;
            [self startTimeoutTracking];
            [_transactionLifecycleAdapter.delegate showProgress];
            
            if (resolvedCode == STARTED_TRANSACTION_3DS){
                [_transactionLifecycleAdapter.delegate threeDsSubmitted];
            }
            
            break;
        case TRANSACTION_SUCCESS:
            _transactionInProgress = false;
            [self stopTimeoutTracking];
            [_transactionLifecycleAdapter.delegate hideProgress];
            [_transactionLifecycleAdapter.delegate transactionSuccess:[TransactionSuccess fromString:inputParam]];
            break;
        case TRANSACTION_FAILURE:{
            _transactionInProgress = false;
            [self stopTimeoutTracking];
            [_transactionLifecycleAdapter.delegate hideProgress];
            NSDictionary* decodedInput = [InputResolver decode:inputParam];
            BusinessError* businessError = [BusinessError fromJson:decodedInput];
            
            if (businessError) {
                [_transactionLifecycleAdapter transactionFailure:businessError];
            } else{
                [_transactionLifecycleAdapter transactionFailure:[ApiError fromJson:decodedInput]];
            }
            
            break;
        }
        case THREE_DS:
            _transactionInProgress = false;
            [self stopTimeoutTracking];
            [_transactionLifecycleAdapter.delegate hideProgress];
            [_transactionLifecycleAdapter.delegate threeDsLoaded];
            break;
            
        case TIMEOUT:
            _transactionInProgress = false;
            [self stopTimeoutTracking];
            [_transactionLifecycleAdapter timeout];
            break;
            
        default:
            NSLog(@"Unable to resolve %d", resolvedCode);
            break;
    }
}

-(void) stopTimeoutTracking{
    if (_timer) {
        [_timer invalidate];
        _timer = nil;
    }
}
-(void) startTimeoutTracking{
    _timer = [NSTimer scheduledTimerWithTimeInterval:1.0
                                              target:self
                                            selector:@selector(onTick:)
                                            userInfo:nil
                                             repeats:YES];
}

-(void)onTick:(NSTimer *)timer {
    NSLog(@"onTick: %ld", _timePassed);
    _timePassed += _timeoutHandlerPeriod;
    if (_timePassed >= _timeout) {
        _timePassed = 0;
        [self notifyDelegate:TIMEOUT request:nil];
    }
}

-(void) dealloc{
    [_timer invalidate];
    _timer = nil;
}
@end
