//
//  TransactionLifecycle.m
//  pay
//
//  Created by Jasmin Suljic on 21/12/16.
//  Copyright © 2016 Leftor d.o.o. All rights reserved.
//

#import <Foundation/Foundation.h>

#import "TransactionError.h"
#import "TransactionSuccess.h"
#import "TransactionLifecycleAdapter.h"
#import "TransactionLifecycleAdapterDelegate.h"
#import "PayModel.h"

int const SHOW_ORDER_RESPONSE = 0;
int const TRANSACTION_SUCCESS_MODE = 1;

@interface TransactionLifecycleAdapter()
@end

@implementation TransactionLifecycleAdapter

-(id) init:(id<TransactionLifecycleAdapterDelegate>)transactionLifecycle api:(PgwApi *)pgwApi merchantKey:(NSString *)key orderNumber:(NSString *)orderNumber{
    return [self init:transactionLifecycle api:pgwApi merchantKey:key orderNumber:orderNumber mode:SHOW_ORDER_RESPONSE];
}

-(id) init:(id<TransactionLifecycleAdapterDelegate>)transactionLifecycle api:(PgwApi *)pgwApi merchantKey:(NSString *)key orderNumber:(NSString *)orderNumber mode:(int)mode{
    self = [super init];
    if (self) {
        self.delegate = transactionLifecycle;
        self.pgwApi = pgwApi;
        self.merchantKey = key;
        self.orderNumber = orderNumber;
        self.mode = mode;
    }
    
    return self;
}

-(void) timeout{
    [_delegate stopWebViewLoad];
    ShowOrderModel *model = [[ShowOrderModel alloc] init: _orderNumber digest: [PayModel sha1:[NSString stringWithFormat:@"%@%@", _merchantKey, _orderNumber]]];
    
    if (_mode == SHOW_ORDER_RESPONSE){
        [_pgwApi checkOrderId:model success:^(ShowOrderResponse *response){
            if (_delegate) {
                [_delegate checkOrderIdSuccess:response];
            }
            
        } failure:^(ApiError *error){
            if(_delegate){
                [_delegate handleTransactionError:error];
            }
        }];
    }else{
        [_pgwApi checkOrder:model success:^(TransactionSuccess *response){
            if (_delegate) {
                [_delegate transactionSuccess:response];
            }
            
        } failure:^(ApiError *error){
            if(_delegate){
                [_delegate handleTransactionError:error];
            }
        }];
    }
    
    
}

-(void) transactionFailure: (ApiError *) error{
    if (error){
        if (_delegate) {
            [_delegate handleTransactionError:error];
        }
    } else{
        if (_delegate) {
            [_delegate handleTransactionError:[ApiErrors libraryParsingError:@"Unable to parse api error"]];
        }
    }
    
}

@end
